// PROJECT  :ResetInterrupt328P
// PURPOSE  :Explore various sources of a System Reset Interrupt on the ATmega328P
// COURSE   :ICS4U
// AUTHOR   :C. D'Arcy
// DATE     :2022 01 04
// MCU      :328P
// STATUS   :Working
// REFERENCE:http://darcy.rsgc.on.ca/ACES/Datasheets/ATMEGA328P.pdf
//          :**see C code on Page 53
#include <Mega328P.h>
// Span a bicolor LED across digital pins 8 and 9 so that green shows with 8 as anode
#define GREEN     (1<<PB0)      // digital pin 8
#define RED       (1<<PB1)      // digital pin 9
#define DURATION  2000          // arbitrary

void setup() {
  // Capture the state of the MCU Status Register
  // Note. This doesn't work as there's too much hidden in the Arduino toolchain before this,
  // but this is the idea...
  uint8_t state = MCUSR;                //grab the contents of the MCUSR as early as possible
  //clear all the available flags (by writing a 1 to them)...      
  MCUSR = 1 << WDRF | 1 << BORF | 1 << EXTRF | 1 << PORF;
  // display the source of the System Reset...
  Serial.begin(9600);
  while (!Serial);
  Serial.println("A System Reset has occurred");      // say a System Reset has occurred...
  Serial.println("MCUSR\t:" + String(state, BIN));    // ?
  Serial.println("PORF\t:" + String(state & (1 << PORF) ? "yes" : "no"));
  Serial.println("EXTRF\t:" + String(state & (1 << EXTRF) ? "yes" : "no"));
  Serial.println("BORF\t:" + String(state & (1 << BORF) ? "yes" : "no"));
  Serial.println("WDRF\t:" + String(state & (1 << WDRF) ? "yes" : "no"));
  Serial.end();

  // signal a Reset event visually by showing green for a short duration...
  DDRB = RED | GREEN;
  PORTB |= GREEN;             //turn green on
  PORTB &= ~RED;              //
  delay(DURATION);            //admire
  PORTB &= ~GREEN;            //turn green off
}
//yeah, nothing to do
void loop() {}
